/*******************************************************************************
* File Name: Bat_Volts.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Bat_Volts.h"

static Bat_Volts_BACKUP_STRUCT  Bat_Volts_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: Bat_Volts_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet Bat_Volts_SUT.c usage_Bat_Volts_Sleep_Wakeup
*******************************************************************************/
void Bat_Volts_Sleep(void)
{
    #if defined(Bat_Volts__PC)
        Bat_Volts_backup.pcState = Bat_Volts_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            Bat_Volts_backup.usbState = Bat_Volts_CR1_REG;
            Bat_Volts_USB_POWER_REG |= Bat_Volts_USBIO_ENTER_SLEEP;
            Bat_Volts_CR1_REG &= Bat_Volts_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(Bat_Volts__SIO)
        Bat_Volts_backup.sioState = Bat_Volts_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        Bat_Volts_SIO_REG &= (uint32)(~Bat_Volts_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: Bat_Volts_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to Bat_Volts_Sleep() for an example usage.
*******************************************************************************/
void Bat_Volts_Wakeup(void)
{
    #if defined(Bat_Volts__PC)
        Bat_Volts_PC = Bat_Volts_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            Bat_Volts_USB_POWER_REG &= Bat_Volts_USBIO_EXIT_SLEEP_PH1;
            Bat_Volts_CR1_REG = Bat_Volts_backup.usbState;
            Bat_Volts_USB_POWER_REG &= Bat_Volts_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(Bat_Volts__SIO)
        Bat_Volts_SIO_REG = Bat_Volts_backup.sioState;
    #endif
}


/* [] END OF FILE */
