/*******************************************************************************
* File Name: CL_DAT.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "CL_DAT.h"

static CL_DAT_BACKUP_STRUCT  CL_DAT_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: CL_DAT_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet CL_DAT_SUT.c usage_CL_DAT_Sleep_Wakeup
*******************************************************************************/
void CL_DAT_Sleep(void)
{
    #if defined(CL_DAT__PC)
        CL_DAT_backup.pcState = CL_DAT_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            CL_DAT_backup.usbState = CL_DAT_CR1_REG;
            CL_DAT_USB_POWER_REG |= CL_DAT_USBIO_ENTER_SLEEP;
            CL_DAT_CR1_REG &= CL_DAT_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(CL_DAT__SIO)
        CL_DAT_backup.sioState = CL_DAT_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        CL_DAT_SIO_REG &= (uint32)(~CL_DAT_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: CL_DAT_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to CL_DAT_Sleep() for an example usage.
*******************************************************************************/
void CL_DAT_Wakeup(void)
{
    #if defined(CL_DAT__PC)
        CL_DAT_PC = CL_DAT_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            CL_DAT_USB_POWER_REG &= CL_DAT_USBIO_EXIT_SLEEP_PH1;
            CL_DAT_CR1_REG = CL_DAT_backup.usbState;
            CL_DAT_USB_POWER_REG &= CL_DAT_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(CL_DAT__SIO)
        CL_DAT_SIO_REG = CL_DAT_backup.sioState;
    #endif
}


/* [] END OF FILE */
