/*******************************************************************************
* File Name: ONSWITCH.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "ONSWITCH.h"

static ONSWITCH_BACKUP_STRUCT  ONSWITCH_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: ONSWITCH_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet ONSWITCH_SUT.c usage_ONSWITCH_Sleep_Wakeup
*******************************************************************************/
void ONSWITCH_Sleep(void)
{
    #if defined(ONSWITCH__PC)
        ONSWITCH_backup.pcState = ONSWITCH_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            ONSWITCH_backup.usbState = ONSWITCH_CR1_REG;
            ONSWITCH_USB_POWER_REG |= ONSWITCH_USBIO_ENTER_SLEEP;
            ONSWITCH_CR1_REG &= ONSWITCH_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(ONSWITCH__SIO)
        ONSWITCH_backup.sioState = ONSWITCH_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        ONSWITCH_SIO_REG &= (uint32)(~ONSWITCH_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: ONSWITCH_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to ONSWITCH_Sleep() for an example usage.
*******************************************************************************/
void ONSWITCH_Wakeup(void)
{
    #if defined(ONSWITCH__PC)
        ONSWITCH_PC = ONSWITCH_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            ONSWITCH_USB_POWER_REG &= ONSWITCH_USBIO_EXIT_SLEEP_PH1;
            ONSWITCH_CR1_REG = ONSWITCH_backup.usbState;
            ONSWITCH_USB_POWER_REG &= ONSWITCH_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(ONSWITCH__SIO)
        ONSWITCH_SIO_REG = ONSWITCH_backup.sioState;
    #endif
}


/* [] END OF FILE */
