/*******************************************************************************
* File Name: SPI_M_EZI2C_INT.c
* Version 1.20
*
* Description:
*  This file provides the source code to the Interrupt Service Routine for
*  the SCB Component in EZI2C mode.
*
* Note:
*
********************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SPI_M_PVT.h"
#include "SPI_M_EZI2C_PVT.h"

#if(SPI_M_EZI2C_SCL_STRETCH_ENABLE_CONST)
    /*******************************************************************************
    * Function Name: SPI_M_EZI2C_STRETCH_ISR
    ********************************************************************************
    *
    * Summary:
    *  Handles the Interrupt Service Routine for the SCB EZI2C mode. The clock stretching is
    *  used during operation.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    CY_ISR_PROTO(SPI_M_EZI2C_STRETCH_ISR)
    {
        static uint16 locBufSize;
        uint32 locIndex;
        uint32 locStatus;

        uint32 endTransfer;
        uint32 fifoIndex;
        uint32 locByte;

        uint32 locIntrCause;
        uint32 locIntrSlave;

    #if(SPI_M_SECONDARY_ADDRESS_ENABLE_CONST)
        /* Variable intended to be used with either buffer */
        static volatile uint8 * SPI_M_dataBuffer; /* Pointer to data buffer              */
        static uint16 SPI_M_bufSizeBuf;           /* Size of buffer1 in bytes            */
        static uint16 SPI_M_protectBuf;           /* Start index of write protected area */

        static uint8 activeAddress;
        uint32 ackResponse;

        ackResponse = SPI_M_EZI2C_ACK_RECEIVED_ADDRESS;
    #endif /* (SPI_M_SECONDARY_ADDRESS_ENABLE_CONST) */

    #if !defined (CY_REMOVE_SPI_M_CUSTOM_INTR_HANDLER)
        if(NULL != SPI_M_customIntrHandler)
        {
            SPI_M_customIntrHandler();
        }
    #else
        CY_SPI_M_CUSTOM_INTR_HANDLER();
    #endif /* !defined (CY_REMOVE_SPI_M_CUSTOM_INTR_HANDLER) */

        /* Make local copy of global variable */
        locIndex = SPI_M_EZI2C_GET_INDEX(activeAddress);

        /* Get interrupt sources */
        locIntrSlave = SPI_M_GetSlaveInterruptSource();
        locIntrCause = SPI_M_GetInterruptCause();

        /* INTR_SLAVE.I2C_ARB_LOST and INTR_SLAVE_I2C.BUS_ERROR */
        /* Handles errors on the bus. There are cases when both bits are set.
        * The error recovery is common: re-enable the scb IP. The content of the RX FIFO is lost.
        */
        if(0u != (locIntrSlave & (SPI_M_INTR_SLAVE_I2C_ARB_LOST |
                                  SPI_M_INTR_SLAVE_I2C_BUS_ERROR)))
        {
            SPI_M_CTRL_REG &= (uint32) ~SPI_M_CTRL_ENABLED; /* Disable SCB block */

        #if(SPI_M_CY_SCBIP_V0)
            if(0u != ((uint8) SPI_M_EZI2C_STATUS_BUSY & SPI_M_curStatus))
        #endif /* (SPI_M_CY_SCBIP_V0) */
            {
                SPI_M_curStatus &= (uint8) ~SPI_M_EZI2C_STATUS_BUSY;
                SPI_M_curStatus |= (uint8)  SPI_M_EZI2C_STATUS_ERR;

                /* INTR_TX_EMPTY is enabled on address phase to receive data */
                if(0u == (SPI_M_GetTxInterruptMode() & SPI_M_INTR_TX_EMPTY))
                {
                    /* Write complete */
                    if(SPI_M_indexBuf1 != SPI_M_offsetBuf1)
                    {
                        SPI_M_curStatus |= (uint8) SPI_M_INTR_SLAVE_I2C_WRITE_STOP;
                    }
                }
                else
                {
                    /* Read complete */
                    SPI_M_curStatus |= (uint8) SPI_M_INTR_SLAVE_I2C_NACK;
                }
            }

            SPI_M_DISABLE_SLAVE_AUTO_DATA;
            
            /* Disable TX and RX interrupt sources */
            SPI_M_SetRxInterruptMode(SPI_M_NO_INTR_SOURCES);
            SPI_M_SetTxInterruptMode(SPI_M_NO_INTR_SOURCES);

        #if(SPI_M_CY_SCBIP_V0)
            /* Clear interrupt sources as they are not automatically cleared after SCB is disabled */
            SPI_M_ClearRxInterruptSource(SPI_M_INTR_RX_ALL);
            SPI_M_ClearSlaveInterruptSource(SPI_M_INTR_SLAVE_ALL);
        #endif /* (SPI_M_CY_SCBIP_V0) */

            SPI_M_fsmState = SPI_M_EZI2C_FSM_IDLE;

            SPI_M_CTRL_REG |= (uint32) SPI_M_CTRL_ENABLED;  /* Enable SCB block */
        }
        else
        {
            /* INTR_I2C_EC_WAKE_UP */
            /* Wakes up device from deep sleep */
            if(0u != (locIntrCause & SPI_M_INTR_CAUSE_I2C_EC))
            {
                /* Disables wakeup interrupt source but do not clear it. It is cleared in INTR_SLAVE_I2C_ADDR_MATCH */
                SPI_M_SetI2CExtClkInterruptMode(SPI_M_NO_INTR_SOURCES);
            }

            if(0u != (locIntrCause & (SPI_M_INTR_CAUSE_RX | SPI_M_INTR_CAUSE_SLAVE)))
            {
                /* INTR_RX.NOT_EMPTY */
                /* Receives data byte-by-byte. Do not use RX FIFO capabilities */
                if (0u != (SPI_M_GetRxInterruptSourceMasked() & SPI_M_INTR_RX_NOT_EMPTY))
                {
                #if (SPI_M_SECONDARY_ADDRESS_ENABLE_CONST)
                    /* If I2C_STOP service is delayed to I2C_ADDR_MATCH the address byte is in the RX FIFO and
                    * RX_NOT_EMPTY is enabled. The address byte has to stay into RX FIFO therefore 
                    * RX.NOT_EMPTY service has to be skipped. The address byte has to be read by I2C_ADDR_MATCH.
                    */
                    if (0u == (locIntrCause & SPI_M_INTR_CAUSE_SLAVE))
                #endif /* (SPI_M_SECONDARY_ADDRESS_ENABLE_CONST) */
                    {
                        locByte = SPI_M_RX_FIFO_RD_REG;

                        switch(SPI_M_fsmState)
                        {
                        case SPI_M_EZI2C_FSM_BYTE_WRITE:
                            if(0u != locBufSize)
                            {
                                /* Store data byte and ACK */
                                SPI_M_I2C_SLAVE_GENERATE_ACK;

                                SPI_M_dataBuffer[locIndex] = (uint8) locByte;
                                locIndex++;
                                locBufSize--;
                            }
                            else
                            {
                                /* Discard data byte and NACK */
                                SPI_M_I2C_SLAVE_GENERATE_NACK;
                            }
                            break;

                    #if(SPI_M_SUB_ADDRESS_SIZE16_CONST)
                        case SPI_M_EZI2C_FSM_OFFSET_HI8:

                            SPI_M_I2C_SLAVE_GENERATE_ACK;

                            /* Store offset most significant byre */
                            locBufSize = (uint16) ((uint8) locByte);

                            SPI_M_fsmState = SPI_M_EZI2C_FSM_OFFSET_LO8;

                            break;
                    #endif /* (SPI_M_SUB_ADDRESS_SIZE16_CONST) */

                        case SPI_M_EZI2C_FSM_OFFSET_LO8:

                            if(SPI_M_SUB_ADDRESS_SIZE16)
                            {
                                /* Collect 2 bytes offset */
                                locByte = ((uint32) ((uint32) locBufSize << 8u)) | locByte;
                            }
                            

                            /* Check offset against buffer size */
                            if(locByte < (uint32) SPI_M_bufSizeBuf)
                            {
                                SPI_M_I2C_SLAVE_GENERATE_ACK;

                                /* Update local buffer index with new offset */
                                locIndex = locByte;

                                /* Get available buffer size to write */
                                locBufSize = (uint16) ((locByte < SPI_M_protectBuf) ?
                                                       (SPI_M_protectBuf - locByte) : (0u));

                            #if (SPI_M_CY_SCBIP_V0)
                                
                                if(locBufSize < SPI_M_FIFO_SIZE)
                                {
                                    /* Set FSM state to receive byte by byte */
                                    SPI_M_fsmState = SPI_M_EZI2C_FSM_BYTE_WRITE;
                                }
                                /* Receive use RX FIFO chunks */
                                else if(locBufSize == SPI_M_FIFO_SIZE)
                                {
                                    SPI_M_ENABLE_SLAVE_AUTO_DATA; /* NACK when RX FIFO is full */
                                    SPI_M_SetRxInterruptMode(SPI_M_NO_INTR_SOURCES);
                                }
                                else
                                {
                                    SPI_M_ENABLE_SLAVE_AUTO_DATA_ACK; /* Stretch when RX FIFO is full */
                                    SPI_M_SetRxInterruptMode(SPI_M_INTR_RX_FULL);
                                }
                                
                            #else
                                
                                if(SPI_M_SECONDARY_ADDRESS_ENABLE)
                                {
                                    /* Set FSM state to receive byte by byte.  
                                    * The byte by byte receive is always chosen for two addreses. Ticket ID#175559.
                                    */
                                    SPI_M_fsmState = SPI_M_EZI2C_FSM_BYTE_WRITE;
                                }
                                else
                                {
                                    if(locBufSize < SPI_M_FIFO_SIZE)
                                    {
                                        /* Set FSM state to receive byte by byte */
                                        SPI_M_fsmState = SPI_M_EZI2C_FSM_BYTE_WRITE;
                                    }
                                    /* Receive use RX FIFO chunks */
                                    else if(locBufSize == SPI_M_FIFO_SIZE)
                                    {
                                        SPI_M_ENABLE_SLAVE_AUTO_DATA; /* NACK when RX FIFO is full */
                                        SPI_M_SetRxInterruptMode(SPI_M_NO_INTR_SOURCES);
                                    }
                                    else
                                    {
                                        SPI_M_ENABLE_SLAVE_AUTO_DATA_ACK; /* Stretch when RX FIFO is full */
                                        SPI_M_SetRxInterruptMode(SPI_M_INTR_RX_FULL);
                                    }
                                }
                                
                                
                            #endif /* (SPI_M_CY_SCBIP_V0) */

                                /* Store local offset into global variable */
                                SPI_M_EZI2C_SET_OFFSET(activeAddress, locIndex);
                            }
                            else
                            {
                                /* Discard offset byte and NACK */
                                SPI_M_I2C_SLAVE_GENERATE_NACK;
                            }
                            break;

                        default:
                            CYASSERT(0u != 0u); /* Should never get there */
                            break;
                        }

                        SPI_M_ClearRxInterruptSource(SPI_M_INTR_RX_NOT_EMPTY);
                    }
                }
                /* INTR_RX.FULL, INTR_SLAVE.I2C_STOP */
                /* Receive use FIFO chunks: auto data ACK is enabled */
                else if (0u != (SPI_M_I2C_CTRL_REG & SPI_M_I2C_CTRL_S_READY_DATA_ACK))
                {
                    /* Slave interrupt (I2C_STOP or I2C_ADDR_MATCH) leads to completion of read. */
                    /* The completion event has a high priority than the FIFO full. Read the remaining data from the RX FIFO. */
                    if(0u != (locIntrCause & SPI_M_INTR_CAUSE_SLAVE))
                    {
                        /* Read remaining bytes from RX FIFO */
                        fifoIndex = SPI_M_GET_RX_FIFO_ENTRIES;

                        if(SPI_M_SECONDARY_ADDRESS_ENABLE)
                        {
                            /* Update with current address match */
                            if(SPI_M_CHECK_INTR_SLAVE_MASKED(SPI_M_INTR_SLAVE_I2C_ADDR_MATCH))
                            {
                                /* Update RX FIFO entries as address byte is there now */
                                fifoIndex = SPI_M_GET_RX_FIFO_ENTRIES;

                                /* If the SR is valid, the RX FIFO is full and the address is in the SHIFTER: read 8 entries and
                                * leave the address in the RX FIFO for further processing.
                                * If the SR is invalid, the address is already in the RX FIFO: read (entries-1).
                                */
                                fifoIndex -= ((0u != SPI_M_GET_RX_FIFO_SR_VALID) ? (0u) : (1u));
                            }
                        }
                        

                        SPI_M_DISABLE_SLAVE_AUTO_DATA;
                        endTransfer = SPI_M_EZI2C_CONTINUE_TRANSFER;
                    }
                    else
                    /* INTR_RX_FULL */
                    /* Continue transfer or disable INTR_RX_FULL to catch completion event. */
                    {
                        /* Calculate buffer size available to write data into */
                        locBufSize -= (uint16) SPI_M_FIFO_SIZE;

                        if(locBufSize <= SPI_M_FIFO_SIZE)
                        {
                            /* Send NACK when RX FIFO overflow */
                            fifoIndex   = locBufSize;
                            endTransfer = SPI_M_EZI2C_COMPLETE_TRANSFER;
                        }
                        else
                        {
                            /* Continue  transaction */
                            fifoIndex   = SPI_M_FIFO_SIZE;
                            endTransfer = SPI_M_EZI2C_CONTINUE_TRANSFER;
                        }
                    }

                    for(; (0u != fifoIndex); fifoIndex--)
                    {
                        /* Store data in the buffer */
                        SPI_M_dataBuffer[locIndex] = (uint8) SPI_M_RX_FIFO_RD_REG;
                        locIndex++;
                    }

                    /* Complete transfer sending NACK when RX FIFO overflow */
                    if(SPI_M_EZI2C_COMPLETE_TRANSFER == endTransfer)
                    {
                        SPI_M_ENABLE_SLAVE_AUTO_DATA_NACK;

                        /* Disable INTR_RX_FULL while last RX FIFO chunk reception */
                        SPI_M_SetRxInterruptMode(SPI_M_NO_INTR_SOURCES);
                    }

                    SPI_M_ClearRxInterruptSource(SPI_M_INTR_RX_FULL |
                                                            SPI_M_INTR_RX_NOT_EMPTY);
                }
                else
                {
                    /* Exit for slave interrupts which do not active for RX direction:
                    * INTR_SLAVE.I2C_ADDR_MATCH and INTR_SLAVE.I2C_STOP while byte-by-byte reception.
                    */
                }
            }

            if(0u != (locIntrCause & SPI_M_INTR_CAUSE_SLAVE))
            {
                /* INTR_SLAVE.I2C_STOP */
                /* Catch Stop condition: completion of write or read transfer */
            #if(!SPI_M_SECONDARY_ADDRESS_ENABLE_CONST)
                if(0u != (locIntrSlave & SPI_M_INTR_SLAVE_I2C_STOP))
            #else
                /* Prevent triggering when matched address was NACKed */
                if((0u != (locIntrSlave & SPI_M_INTR_SLAVE_I2C_STOP)) &&
                   (0u != ((uint8) SPI_M_EZI2C_STATUS_BUSY & SPI_M_curStatus)))
            #endif
                {
                    /* Disable TX and RX interrupt sources */
                    SPI_M_SetRxInterruptMode(SPI_M_NO_INTR_SOURCES);
                    SPI_M_SetTxInterruptMode(SPI_M_NO_INTR_SOURCES);

                    /* Set read completion mask */
                    locStatus = SPI_M_INTR_SLAVE_I2C_NACK;

                    /* Check if buffer content was modified: the address phase resets the locIndex */
                    if(locIndex != SPI_M_EZI2C_GET_OFFSET(activeAddress))
                    {
                        locStatus |= SPI_M_INTR_SLAVE_I2C_WRITE_STOP;
                    }

                    /* Complete read or write transaction */
                    locStatus &= locIntrSlave;
                    SPI_M_EZI2C_UPDATE_LOC_STATUS(activeAddress, locStatus);
                    locStatus |= (uint32)  SPI_M_curStatus;
                    locStatus &= (uint32) ~SPI_M_EZI2C_STATUS_BUSY;
                    SPI_M_curStatus = (uint8) locStatus;

                    SPI_M_fsmState = SPI_M_EZI2C_FSM_IDLE;

                    if(SPI_M_SECONDARY_ADDRESS_ENABLE)
                    {
                        /* Store local index into global variable, before addrss phase */
                        SPI_M_EZI2C_SET_INDEX(activeAddress, locIndex);
                    }
                    
                }

                /* INTR_SLAVE.I2C_ADDR_MATCH */
                /* The matched address is received: the slave starts its operation.
                * The INTR_SLAVE_I2C_STOP updates the buffer index before the address phase for two addresses mode.
                * This is done to update buffer index correctly before the address phase changes it.
                */
                if(0u != (locIntrSlave & SPI_M_INTR_SLAVE_I2C_ADDR_MATCH))
                {
                    if(SPI_M_SECONDARY_ADDRESS_ENABLE)
                    {
                        /* Read address byte from RX FIFO */
                        locByte = SPI_M_GET_I2C_7BIT_ADDRESS(SPI_M_RX_FIFO_RD_REG);

                        SPI_M_ClearRxInterruptSource(SPI_M_INTR_RX_NOT_EMPTY);

                        /* Check received address against device addresses */
                        if(SPI_M_addrBuf1 == locByte)
                        {
                            /* Set buffer exposed to primary slave address */
                            SPI_M_dataBuffer = SPI_M_dataBuffer1;
                            SPI_M_bufSizeBuf = SPI_M_bufSizeBuf1;
                            SPI_M_protectBuf = SPI_M_protectBuf1;

                            activeAddress = SPI_M_EZI2C_ACTIVE_ADDRESS1;
                        }
                        else if(SPI_M_addrBuf2 == locByte)
                        {
                            /* Set buffer exposed to secondary slave address */
                            SPI_M_dataBuffer = SPI_M_dataBuffer2;
                            SPI_M_bufSizeBuf = SPI_M_bufSizeBuf2;
                            SPI_M_protectBuf = SPI_M_protectBuf2;

                            activeAddress = SPI_M_EZI2C_ACTIVE_ADDRESS2;
                        }
                        else
                        {
                            /* Address does not match */
                            ackResponse = SPI_M_EZI2C_NACK_RECEIVED_ADDRESS;
                        }
                    }
                    

                #if(SPI_M_SECONDARY_ADDRESS_ENABLE_CONST)
                    if(SPI_M_EZI2C_NACK_RECEIVED_ADDRESS == ackResponse)
                    {
                        /* Clear interrupt sources before NACK address */
                        SPI_M_ClearI2CExtClkInterruptSource(SPI_M_INTR_I2C_EC_WAKE_UP);
                        SPI_M_ClearSlaveInterruptSource(SPI_M_INTR_SLAVE_ALL);

                    #if(!SPI_M_CY_SCBIP_V0)
                        /* Disable INTR_I2C_STOP to not trigger after matched address is NACKed. Ticket ID#156094 */
                        SPI_M_DISABLE_INTR_SLAVE(SPI_M_INTR_SLAVE_I2C_STOP);
                    #endif /* (!SPI_M_CY_SCBIP_V0) */

                        /* NACK address byte: it does not match neither primary nor secondary */
                        SPI_M_I2C_SLAVE_GENERATE_NACK;
                    }
                    else
                #endif /* (SPI_M_SECONDARY_ADDRESS_ENABLE_CONST) */
                    {

                    #if(SPI_M_SCB_MODE_UNCONFIG_CONST_CFG)
                        if(!SPI_M_SECONDARY_ADDRESS_ENABLE)
                        {
                            /* Set buffer exposed to primary slave address */
                            SPI_M_dataBuffer = SPI_M_dataBuffer1;
                            SPI_M_bufSizeBuf = SPI_M_bufSizeBuf1;
                            SPI_M_protectBuf = SPI_M_protectBuf1;

                            activeAddress = SPI_M_EZI2C_ACTIVE_ADDRESS1;
                        }
                    #endif /* (SPI_M_SCB_MODE_UNCONFIG_CONST_CFG) */

                        /* Bus becomes busy after address is received */
                        SPI_M_curStatus |= (uint8) SPI_M_EZI2C_STATUS_BUSY;

                        /* Slave is read or written: set current offset */
                        locIndex = SPI_M_EZI2C_GET_OFFSET(activeAddress);

                        /* Check transaction direction */
                        if(SPI_M_CHECK_I2C_STATUS(SPI_M_I2C_STATUS_S_READ))
                        {
                            /* Calculate slave buffer size */
                            locBufSize = SPI_M_bufSizeBuf - (uint16) locIndex;

                            /* Clear TX FIFO to start fill from offset */
                            SPI_M_CLEAR_TX_FIFO;
                            SPI_M_SetTxInterruptMode(SPI_M_INTR_TX_EMPTY);
                        }
                        else
                        {
                            /* Master writes: enable reception interrupt. The FSM state was set in INTR_SLAVE_I2C_STOP */
                            SPI_M_SetRxInterruptMode(SPI_M_INTR_RX_NOT_EMPTY);
                        }

                        /* Clear interrupt sources before ACK address */
                        SPI_M_ClearI2CExtClkInterruptSource(SPI_M_INTR_I2C_EC_WAKE_UP);
                        SPI_M_ClearSlaveInterruptSource(SPI_M_INTR_SLAVE_ALL);

                    #if(!SPI_M_CY_SCBIP_V0)
                        /* Enable STOP to trigger after address match is ACKed. Ticket ID#156094 */
                        SPI_M_ENABLE_INTR_SLAVE(SPI_M_INTR_SLAVE_I2C_STOP);
                    #endif /* (!SPI_M_CY_SCBIP_V0) */

                        /* ACK the address byte */
                        SPI_M_I2C_SLAVE_GENERATE_ACK;
                    }
                }

                /* Clear slave interrupt sources */
                SPI_M_ClearSlaveInterruptSource(locIntrSlave);
            }

            /* INTR_TX.EMPTY */
            /* Transmits data to the master: loads data into the TX FIFO. The 0xFF sends out if the master reads? out the buffer.
            * The address reception with a read flag clears the TX FIFO to be loaded with data.
            */
            if(0u != (SPI_M_GetInterruptCause() & SPI_M_INTR_CAUSE_TX))
            {
                /* Put data into TX FIFO until there is a room */
                do
                {
                    /* Check transmit buffer range: locBufSize calculates after address reception */
                    if(0u != locBufSize)
                    {
                        SPI_M_TX_FIFO_WR_REG = (uint32) SPI_M_dataBuffer[locIndex];
                        locIndex++;
                        locBufSize--;
                    }
                    else
                    {
                        SPI_M_TX_FIFO_WR_REG = SPI_M_EZI2C_OVFL_RETURN;
                    }
                }
                while(SPI_M_FIFO_SIZE != SPI_M_GET_TX_FIFO_ENTRIES);

                SPI_M_ClearTxInterruptSource(SPI_M_INTR_TX_EMPTY);
            }
        }

        /* Store local index copy into global variable */
        SPI_M_EZI2C_SET_INDEX(activeAddress, locIndex);
    }
#endif /* (SPI_M_EZI2C_SCL_STRETCH_ENABLE_CONST) */


#if(SPI_M_EZI2C_SCL_STRETCH_DISABLE_CONST)
    /*******************************************************************************
    * Function Name: SPI_M_EZI2C_NO_STRETCH_ISR
    ********************************************************************************
    *
    * Summary:
    *  Handles the Interrupt Service Routine for the SCB EZI2C mode. Clock stretching is
    *  NOT used during operation.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    CY_ISR_PROTO(SPI_M_EZI2C_NO_STRETCH_ISR)
    {
    #if(SPI_M_SUB_ADDRESS_SIZE16_CONST)
        static uint8 locOffset;
    #endif /* (SPI_M_SUB_ADDRESS_SIZE16_CONST) */

        uint32 locByte;
        uint32 locStatus;
        uint32 locIntrSlave;
        uint32 locIntrCause;

    #if !defined (CY_REMOVE_SPI_M_CUSTOM_INTR_HANDLER)
        /* Calls registered customer routine to manage interrupt sources */
        if(NULL != SPI_M_customIntrHandler)
        {
            SPI_M_customIntrHandler();
        }
    #else
        CY_SPI_M_CUSTOM_INTR_HANDLER();
    #endif /* !defined (CY_REMOVE_SPI_M_CUSTOM_INTR_HANDLER) */

        locByte = 0u;

        /* Get copy of triggered slave interrupt sources */
        locIntrSlave = SPI_M_GetSlaveInterruptSource();
        locIntrCause = SPI_M_GetInterruptCause();

        /* INTR_SLAVE.I2C_ARB_LOST and INTR_SLAVE.I2C_BUS_ERROR */
        /* Handles errors on the bus: There are cases when both bits are set.
        * The error recovery is common: re-enable the scb IP. The content of the RX FIFO is lost.
        */
        if(0u != (locIntrSlave & (SPI_M_INTR_SLAVE_I2C_ARB_LOST |
                                  SPI_M_INTR_SLAVE_I2C_BUS_ERROR)))
        {
            SPI_M_CTRL_REG &= (uint32) ~SPI_M_CTRL_ENABLED; /* Disable SCB block */

        #if(SPI_M_CY_SCBIP_V0)
            if(0u != (locIntrSlave & SPI_M_INTR_SLAVE_I2C_ADDR_MATCH))
        #endif /* (SPI_M_CY_SCBIP_V0) */
            {
                SPI_M_curStatus |= (uint8) SPI_M_EZI2C_STATUS_ERR;

                if(0u != (SPI_M_EZI2C_FSM_WRITE_MASK & SPI_M_fsmState))
                {
                    /* Write complete */
                    if(SPI_M_indexBuf1 != SPI_M_offsetBuf1)
                    {
                        SPI_M_curStatus |= (uint8) SPI_M_INTR_SLAVE_I2C_WRITE_STOP;
                    }
                }
                else
                {
                    /* Read complete */
                    SPI_M_curStatus |= (uint8) SPI_M_INTR_SLAVE_I2C_NACK;
                }
            }

            /* Clean-up interrupt sources */
            SPI_M_SetTxInterruptMode(SPI_M_NO_INTR_SOURCES);

        #if(SPI_M_CY_SCBIP_V0)
            /* Clear interrupt sources as they are not automatically cleared after SCB is disabled */
            SPI_M_ClearRxInterruptSource(SPI_M_INTR_RX_NOT_EMPTY);
            SPI_M_ClearSlaveInterruptSource(SPI_M_INTR_SLAVE_ALL);
        #endif /* (SPI_M_CY_SCBIP_V0) */

            SPI_M_fsmState = SPI_M_EZI2C_FSM_IDLE;

            SPI_M_CTRL_REG |= (uint32) SPI_M_CTRL_ENABLED;  /* Enable SCB block */
        }
        else
        {
            /* INTR_RX.NOT_EMPTY */
            /* The slave receives data from the master: accepts into the RX FIFO. At least one entry is available to be read.
            * The offset is written first and all the following bytes are data (expected to be put in the buffer).
            * The slave ACKs all bytes, but it discards them if they do not match the write criteria.
            * the slave NACKs the bytes in case of the RX FIFO overflow.
            */
            if(0u != (locIntrCause & SPI_M_INTR_CAUSE_RX))
            {
                /* Read all entries available in RX FIFO */
                do
                {
                    locByte = SPI_M_RX_FIFO_RD_REG;

                    switch(SPI_M_fsmState)
                    {

                    case SPI_M_EZI2C_FSM_BYTE_WRITE:
                        /* Check buffer index against protect area */
                        if(SPI_M_indexBuf1 < SPI_M_protectBuf1)
                        {
                            /* Stores received byte into the buffer */
                            SPI_M_dataBuffer1[SPI_M_indexBuf1] = (uint8) locByte;
                            SPI_M_indexBuf1++;
                        }
                        else
                        {
                            /* Discard current byte and sets FSM state to discard following bytes */
                            SPI_M_fsmState = SPI_M_EZI2C_FSM_WAIT_STOP;
                        }

                        break;

                #if(SPI_M_SUB_ADDRESS_SIZE16_CONST)
                    case SPI_M_EZI2C_FSM_OFFSET_HI8:

                        /* Store high byte of offset */
                        locOffset = (uint8) locByte;

                        SPI_M_fsmState  = SPI_M_EZI2C_FSM_OFFSET_LO8;

                        break;
                #endif /* (SPI_M_SUB_ADDRESS_SIZE16_CONST) */

                    case SPI_M_EZI2C_FSM_OFFSET_LO8:

                        if(SPI_M_SUB_ADDRESS_SIZE16)
                        {
                            /* Append the offset with high byte */
                            locByte = ((uint32) ((uint32) locOffset << 8u)) | locByte;
                        }
                        

                        /* Check if offset within buffer range */
                        if(locByte < (uint32) SPI_M_bufSizeBuf1)
                        {
                            /* Store and sets received offset */
                            SPI_M_offsetBuf1 = (uint16) locByte;
                            SPI_M_indexBuf1  = (uint16) locByte;

                            /* Move FSM to data receive state */
                            SPI_M_fsmState = SPI_M_EZI2C_FSM_BYTE_WRITE;
                        }
                        else
                        {
                            /* Reset index due to TX FIFO fill */
                            SPI_M_indexBuf1 = (uint16) SPI_M_offsetBuf1;

                            /* Discard current byte and sets FSM state to default to discard following bytes */
                            SPI_M_fsmState = SPI_M_EZI2C_FSM_WAIT_STOP;
                        }

                        break;

                    case SPI_M_EZI2C_FSM_WAIT_STOP:
                        /* Clear RX FIFO to discard all received data */
                        SPI_M_CLEAR_RX_FIFO;

                        break;

                    default:
                        CYASSERT(0u != 0u); /* Should never get there */
                        break;
                    }

                }
                while(0u != SPI_M_GET_RX_FIFO_ENTRIES);

                SPI_M_ClearRxInterruptSource(SPI_M_INTR_RX_NOT_EMPTY);
            }


            /* INTR_SLAVE.I2C_START */
            /* Catches start of transfer to trigger TX FIFO update event */
            if(0u != (locIntrSlave & SPI_M_INTR_SLAVE_I2C_START))
            {
            #if(!SPI_M_CY_SCBIP_V0)
                if(SPI_M_EZI2C_EC_AM_ENABLE)
                {
                    /* Manage INTR_I2C_EC.WAKE_UP as slave busy status */
                    SPI_M_ClearI2CExtClkInterruptSource(SPI_M_INTR_I2C_EC_WAKE_UP);
                }
                else
                {
                    /* Manage INTR_SLAVE.I2C_ADDR_MATCH as slave busy status */
                    SPI_M_ClearSlaveInterruptSource(SPI_M_INTR_SLAVE_I2C_ADDR_MATCH);
                }
                
            #else
                /* Manage INTR_SLAVE.I2C_ADDR_MATCH as slave busy status */
                SPI_M_ClearSlaveInterruptSource(SPI_M_INTR_SLAVE_I2C_ADDR_MATCH);
            #endif /* (SPI_M_CY_SCBIP_V0) */

                /* Clear TX FIFO and put a byte */
                SPI_M_CLEAR_TX_FIFO;
                SPI_M_TX_FIFO_WR_REG = (uint32) SPI_M_dataBuffer1[SPI_M_offsetBuf1];

                /* Store buffer index to be handled by INTR_SLAVE.I2C_STOP */
                locByte = (uint32) SPI_M_indexBuf1;

                /* Update index: one byte is already in the TX FIFO */
                SPI_M_indexBuf1 = (uint16) SPI_M_offsetBuf1 + 1u;

                /* Enable INTR_TX.NOT_FULL to load TX FIFO */
                SPI_M_SetTxInterruptMode(SPI_M_INTR_TX_TRIGGER);

                /* Clear locIntrSlave after INTR.TX_TRIGGER is enabled */
                SPI_M_ClearSlaveInterruptSource(locIntrSlave);

                locIntrCause |= SPI_M_INTR_CAUSE_TX;
            }


            /* INTR_TX.NOT_FULL */
            /* Transmits data to the master: loads data into the TX FIFO. At least one empty entry in the TX FIFO available to load.
            * If there is data to transmit, the 0xFF send out.
            * At the Start of any transfer the TX FIFO is loaded with data from the current buffer offset.
            * This action is intended to reduce probability of clock stretching if the TX FIFO and SHIRTER are empty.
            */
            if(0u != (locIntrCause & SPI_M_INTR_CAUSE_TX))
            {
                /* Put data into TX FIFO until there is a room */
                do
                {
                    /* Check transmit buffer range */
                    if(SPI_M_indexBuf1 < SPI_M_bufSizeBuf1)
                    {
                        SPI_M_TX_FIFO_WR_REG = (uint32) SPI_M_dataBuffer1[SPI_M_indexBuf1];
                        SPI_M_indexBuf1++;
                    }
                    else
                    {
                        SPI_M_TX_FIFO_WR_REG = SPI_M_EZI2C_OVFL_RETURN;
                    }

                }
                while(SPI_M_TX_LOAD_SIZE != SPI_M_GET_TX_FIFO_ENTRIES);

                SPI_M_ClearTxInterruptSource(SPI_M_INTR_TX_TRIGGER);
            }


            /* INTR_SLAVE.I2C_STOP */
            /* Catch completion of write or read transfer. */
            if(0u != (locIntrSlave & SPI_M_INTR_SLAVE_I2C_STOP))
            {
                if(0u == (locIntrSlave & SPI_M_INTR_SLAVE_I2C_START))
                {
                #if(!SPI_M_CY_SCBIP_V0)
                    if(SPI_M_EZI2C_EC_AM_ENABLE)
                    {
                        /* Manage INTR_I2C_EC.WAKE_UP as slave busy status */
                        SPI_M_ClearI2CExtClkInterruptSource(SPI_M_INTR_I2C_EC_WAKE_UP);
                    }
                    
                #endif /* (!SPI_M_CY_SCBIP_V0) */

                    /* Manage INTR_SLAVE.I2C_ADDR_MATCH as slave busy status */
                    SPI_M_ClearSlaveInterruptSource(locIntrSlave);

                    /* Read current buffer index */
                    locByte = (uint32) SPI_M_indexBuf1;
                }

                /* Set read completion mask */
                locStatus = SPI_M_INTR_SLAVE_I2C_NACK;

                if((locByte != SPI_M_offsetBuf1) &&
                   (0u != (SPI_M_EZI2C_FSM_WRITE_MASK & SPI_M_fsmState)))
                {
                    /* Set write completion mask */
                    locStatus |= SPI_M_INTR_SLAVE_I2C_WRITE_STOP;
                }

                /* Set completion flags in the status variable */
                SPI_M_curStatus |= (uint8) (locStatus & locIntrSlave);

                SPI_M_fsmState = SPI_M_EZI2C_FSM_IDLE;
            }


        #if(!SPI_M_CY_SCBIP_V0)
            if(SPI_M_EZI2C_EC_AM_ENABLE)
            {
                /* INTR_I2C_EC.WAKE_UP */
                /* Wake up device from deep sleep on address match event. The matched address is NACKed */
                if(0u != (locIntrCause & SPI_M_INTR_CAUSE_I2C_EC))
                {
                    SPI_M_I2C_SLAVE_GENERATE_NACK; /* NACK in active mode */
                    SPI_M_ClearI2CExtClkInterruptSource(SPI_M_INTR_I2C_EC_WAKE_UP);
                }
            }
            
        #endif /* (!SPI_M_CY_SCBIP_V0) */
        }
    }
#endif /* (SPI_M_EZI2C_SCL_STRETCH_DISABLE_CONST) */


/* [] END OF FILE */
