/*******************************************************************************
* File Name: SPI_M_I2C.c
* Version 1.20
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  I2C mode.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SPI_M_PVT.h"
#include "SPI_M_I2C_PVT.h"


/***************************************
*      I2C Private Vars
***************************************/

volatile uint8 SPI_M_state;  /* Current state of I2C FSM */


#if(SPI_M_SCB_MODE_UNCONFIG_CONST_CFG)

    /***************************************
    *  Config Structure Initialization
    ***************************************/

    /* Constant configuration of I2C */
    const SPI_M_I2C_INIT_STRUCT SPI_M_configI2C =
    {
        SPI_M_I2C_MODE,
        SPI_M_I2C_OVS_FACTOR_LOW,
        SPI_M_I2C_OVS_FACTOR_HIGH,
        SPI_M_I2C_MEDIAN_FILTER_ENABLE,
        SPI_M_I2C_SLAVE_ADDRESS,
        SPI_M_I2C_SLAVE_ADDRESS_MASK,
        SPI_M_I2C_ACCEPT_ADDRESS,
        SPI_M_I2C_WAKE_ENABLE
    };

    /*******************************************************************************
    * Function Name: SPI_M_I2CInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for I2C operation.
    *
    * Parameters:
    *  config:  Pointer to a structure that contains the following ordered list of
    *           fields. These fields match the selections available in the
    *           customizer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SPI_M_I2CInit(const SPI_M_I2C_INIT_STRUCT *config)
    {
        if(NULL == config)
        {
            CYASSERT(0u != 0u); /* Halt execution due bad function parameter */
        }
        else
        {
            /* Configure pins */
            SPI_M_SetPins(SPI_M_SCB_MODE_I2C, SPI_M_DUMMY_PARAM,
                                                                    SPI_M_DUMMY_PARAM);

            /* Store internal configuration */
            SPI_M_scbMode       = (uint8) SPI_M_SCB_MODE_I2C;
            SPI_M_scbEnableWake = (uint8) config->enableWake;
            SPI_M_scbEnableIntr = (uint8) SPI_M_SCB_IRQ_INTERNAL;

            SPI_M_mode          = (uint8) config->mode;
            SPI_M_acceptAddr    = (uint8) config->acceptAddr;

            /* Configure I2C interface */
            SPI_M_CTRL_REG     = SPI_M_GET_CTRL_ADDR_ACCEPT(config->acceptAddr) |
                                            SPI_M_GET_CTRL_EC_AM_MODE (config->enableWake);

            SPI_M_I2C_CTRL_REG = SPI_M_GET_I2C_CTRL_HIGH_PHASE_OVS(config->oversampleHigh) |
                                            SPI_M_GET_I2C_CTRL_LOW_PHASE_OVS (config->oversampleLow)  |
                                            SPI_M_GET_I2C_CTRL_SL_MSTR_MODE  (config->mode)           |
                                            SPI_M_I2C_CTRL;

        #if(SPI_M_CY_SCBIP_V0)
            /* Adjust SDA filter settings. Ticket ID#150521 */
            SPI_M_SET_I2C_CFG_SDA_FILT_TRIM(SPI_M_EC_AM_I2C_CFG_SDA_FILT_TRIM);
        #endif /* (SPI_M_CY_SCBIP_V0) */

            /* Configure RX direction */
            SPI_M_RX_CTRL_REG      = SPI_M_GET_RX_CTRL_MEDIAN(config->enableMedianFilter) |
                                                SPI_M_I2C_RX_CTRL;
            SPI_M_RX_FIFO_CTRL_REG = SPI_M_CLEAR_REG;

            /* Set default address and mask */
            SPI_M_RX_MATCH_REG    = ((SPI_M_I2C_SLAVE) ?
                                                (SPI_M_GET_I2C_8BIT_ADDRESS(config->slaveAddr) |
                                                 SPI_M_GET_RX_MATCH_MASK(config->slaveAddrMask)) :
                                                (SPI_M_CLEAR_REG));


            /* Configure TX direction */
            SPI_M_TX_CTRL_REG      = SPI_M_I2C_TX_CTRL;
            SPI_M_TX_FIFO_CTRL_REG = SPI_M_CLEAR_REG;

            /* Configure interrupt with I2C handler but do not enable it */
            CyIntDisable    (SPI_M_ISR_NUMBER);
            CyIntSetPriority(SPI_M_ISR_NUMBER, SPI_M_ISR_PRIORITY);
            (void) CyIntSetVector(SPI_M_ISR_NUMBER, &SPI_M_I2C_ISR);

            /* Configure interrupt sources */
        #if(!SPI_M_CY_SCBIP_V1_I2C_ONLY)
            SPI_M_INTR_SPI_EC_MASK_REG = SPI_M_NO_INTR_SOURCES;
        #endif /* (!SPI_M_CY_SCBIP_V1_I2C_ONLY) */

            SPI_M_INTR_I2C_EC_MASK_REG = SPI_M_NO_INTR_SOURCES;
            SPI_M_INTR_RX_MASK_REG     = SPI_M_NO_INTR_SOURCES;
            SPI_M_INTR_TX_MASK_REG     = SPI_M_NO_INTR_SOURCES;

            SPI_M_INTR_SLAVE_MASK_REG  = ((SPI_M_I2C_SLAVE) ?
                                                     (SPI_M_I2C_INTR_SLAVE_MASK) :
                                                     (SPI_M_CLEAR_REG));

            SPI_M_INTR_MASTER_MASK_REG = ((SPI_M_I2C_MASTER) ?
                                                     (SPI_M_I2C_INTR_MASTER_MASK) :
                                                     (SPI_M_CLEAR_REG));

            /* Configure global variables */
            SPI_M_state = SPI_M_I2C_FSM_IDLE;

            /* Internal slave variables */
            SPI_M_slStatus        = 0u;
            SPI_M_slRdBufIndex    = 0u;
            SPI_M_slWrBufIndex    = 0u;
            SPI_M_slOverFlowCount = 0u;

            /* Internal master variables */
            SPI_M_mstrStatus     = 0u;
            SPI_M_mstrRdBufIndex = 0u;
            SPI_M_mstrWrBufIndex = 0u;
        }
    }

#else

    /*******************************************************************************
    * Function Name: SPI_M_I2CInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for the I2C operation.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SPI_M_I2CInit(void)
    {
        /* Configure I2C interface */
        SPI_M_CTRL_REG     = SPI_M_I2C_DEFAULT_CTRL;
        SPI_M_I2C_CTRL_REG = SPI_M_I2C_DEFAULT_I2C_CTRL;

    #if(SPI_M_CY_SCBIP_V0)
        /* Adjust SDA filter settings. Ticket ID#150521 */
        SPI_M_SET_I2C_CFG_SDA_FILT_TRIM(SPI_M_EC_AM_I2C_CFG_SDA_FILT_TRIM);
    #endif /* (SPI_M_CY_SCBIP_V0) */

        /* Configure RX direction */
        SPI_M_RX_CTRL_REG      = SPI_M_I2C_DEFAULT_RX_CTRL;
        SPI_M_RX_FIFO_CTRL_REG = SPI_M_I2C_DEFAULT_RX_FIFO_CTRL;

        /* Set default address and mask */
        SPI_M_RX_MATCH_REG     = SPI_M_I2C_DEFAULT_RX_MATCH;

        /* Configure TX direction */
        SPI_M_TX_CTRL_REG      = SPI_M_I2C_DEFAULT_TX_CTRL;
        SPI_M_TX_FIFO_CTRL_REG = SPI_M_I2C_DEFAULT_TX_FIFO_CTRL;

        /* Configure interrupt with I2C handler but do not enable it */
        CyIntDisable    (SPI_M_ISR_NUMBER);
        CyIntSetPriority(SPI_M_ISR_NUMBER, SPI_M_ISR_PRIORITY);
    #if(!SPI_M_I2C_EXTERN_INTR_HANDLER)
        (void) CyIntSetVector(SPI_M_ISR_NUMBER, &SPI_M_I2C_ISR);
    #endif /* (SPI_M_I2C_EXTERN_INTR_HANDLER) */

        /* Configure interrupt sources */
    #if(!SPI_M_CY_SCBIP_V1_I2C_ONLY)
        SPI_M_INTR_SPI_EC_MASK_REG = SPI_M_I2C_DEFAULT_INTR_SPI_EC_MASK;
    #endif /* (!SPI_M_CY_SCBIP_V1_I2C_ONLY) */

        SPI_M_INTR_I2C_EC_MASK_REG = SPI_M_I2C_DEFAULT_INTR_I2C_EC_MASK;
        SPI_M_INTR_SLAVE_MASK_REG  = SPI_M_I2C_DEFAULT_INTR_SLAVE_MASK;
        SPI_M_INTR_MASTER_MASK_REG = SPI_M_I2C_DEFAULT_INTR_MASTER_MASK;
        SPI_M_INTR_RX_MASK_REG     = SPI_M_I2C_DEFAULT_INTR_RX_MASK;
        SPI_M_INTR_TX_MASK_REG     = SPI_M_I2C_DEFAULT_INTR_TX_MASK;

        /* Configure global variables */
        SPI_M_state = SPI_M_I2C_FSM_IDLE;

    #if(SPI_M_I2C_SLAVE)
        /* Internal slave variable */
        SPI_M_slStatus        = 0u;
        SPI_M_slRdBufIndex    = 0u;
        SPI_M_slWrBufIndex    = 0u;
        SPI_M_slOverFlowCount = 0u;
    #endif /* (SPI_M_I2C_SLAVE) */

    #if(SPI_M_I2C_MASTER)
    /* Internal master variable */
        SPI_M_mstrStatus     = 0u;
        SPI_M_mstrRdBufIndex = 0u;
        SPI_M_mstrWrBufIndex = 0u;
    #endif /* (SPI_M_I2C_MASTER) */
    }
#endif /* (SPI_M_SCB_MODE_UNCONFIG_CONST_CFG) */


/*******************************************************************************
* Function Name: SPI_M_I2CStop
********************************************************************************
*
* Summary:
*  Resets the I2C FSM into the default state and disables TX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*
*
*******************************************************************************/
void SPI_M_I2CStop(void)
{
    /* Disable TX interrupt sources in order not to cause a false trigger.
    * The incoming transaction will enable an appropriate TX interrupt.
    */
    SPI_M_SetTxInterruptMode(SPI_M_NO_INTR_SOURCES);

#if(SPI_M_CY_SCBIP_V0)
    /* Clear pending interrupt as TX FIFO becomes empty and block does not gate interrupt trigger when disabled */
    SPI_M_ClearPendingInt();
#endif /* (SPI_M_CY_SCBIP_V0) */

    SPI_M_state = SPI_M_I2C_FSM_IDLE;
}


#if(SPI_M_I2C_WAKE_ENABLE_CONST)
    /*******************************************************************************
    * Function Name: SPI_M_I2CSaveConfig
    ********************************************************************************
    *
    * Summary:
    *  Enables SPI_M_INTR_I2C_EC_WAKE_UP interrupt source. This interrupt
    *  triggers on address match and wakes up device.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SPI_M_I2CSaveConfig(void)
    {
        SPI_M_SetI2CExtClkInterruptMode(SPI_M_INTR_I2C_EC_WAKE_UP);
    }


    /*******************************************************************************
    * Function Name: SPI_M_I2CRestoreConfig
    ********************************************************************************
    *
    * Summary:
    *  Added for compatibility.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SPI_M_I2CRestoreConfig(void)
    {
        /* SPI_M_INTR_I2C_EC_WAKE_UP is masked-off in the interrupt */
    }
#endif /* (SPI_M_I2C_WAKE_ENABLE_CONST) */


/* [] END OF FILE */
