/*******************************************************************************
* File Name: SPI_M_PM.c
* Version 1.20
*
* Description:
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SPI_M.h"
#include "SPI_M_PVT.h"

#if(SPI_M_SCB_MODE_I2C_INC)
    #include "SPI_M_I2C_PVT.h"
#endif /* (SPI_M_SCB_MODE_I2C_INC) */

#if(SPI_M_SCB_MODE_EZI2C_INC)
    #include "SPI_M_EZI2C_PVT.h"
#endif /* (SPI_M_SCB_MODE_EZI2C_INC) */

#if(SPI_M_SCB_MODE_SPI_INC || SPI_M_SCB_MODE_UART_INC)
    #include "SPI_M_SPI_UART_PVT.h"
#endif /* (SPI_M_SCB_MODE_SPI_INC || SPI_M_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(SPI_M_SCB_MODE_UNCONFIG_CONST_CFG || \
   (SPI_M_SCB_MODE_I2C_CONST_CFG   && (!SPI_M_I2C_WAKE_ENABLE_CONST))   || \
   (SPI_M_SCB_MODE_EZI2C_CONST_CFG && (!SPI_M_EZI2C_WAKE_ENABLE_CONST)) || \
   (SPI_M_SCB_MODE_SPI_CONST_CFG   && (!SPI_M_SPI_WAKE_ENABLE_CONST))   || \
   (SPI_M_SCB_MODE_UART_CONST_CFG  && (!SPI_M_UART_WAKE_ENABLE_CONST)))

    SPI_M_BACKUP_STRUCT SPI_M_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: SPI_M_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component to enter Deep Sleep.
*  The Enable wakeup from Sleep Mode selection has an influence on
*  this function implementation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SPI_M_Sleep(void)
{
#if(SPI_M_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SPI_M_SCB_WAKE_ENABLE_CHECK)
    {
        if(SPI_M_SCB_MODE_I2C_RUNTM_CFG)
        {
            SPI_M_I2CSaveConfig();
        }
        else if(SPI_M_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SPI_M_EzI2CSaveConfig();
        }
    #if(!SPI_M_CY_SCBIP_V1_I2C_ONLY)
        else if(SPI_M_SCB_MODE_SPI_RUNTM_CFG)
        {
            SPI_M_SpiSaveConfig();
        }
        else if(SPI_M_SCB_MODE_UART_RUNTM_CFG)
        {
            SPI_M_UartSaveConfig();
        }
    #endif /* (!SPI_M_CY_SCBIP_V1_I2C_ONLY) */
        else
        {
            /* Unknown mode: do nothing */
        }
    }
    else
    {
        SPI_M_backup.enableState = (uint8) SPI_M_GET_CTRL_ENABLED;

        if(0u != SPI_M_backup.enableState)
        {
            SPI_M_Stop();
        }
    }

#else

    #if (SPI_M_SCB_MODE_I2C_CONST_CFG && SPI_M_I2C_WAKE_ENABLE_CONST)
        SPI_M_I2CSaveConfig();

    #elif (SPI_M_SCB_MODE_EZI2C_CONST_CFG && SPI_M_EZI2C_WAKE_ENABLE_CONST)
        SPI_M_EzI2CSaveConfig();

    #elif (SPI_M_SCB_MODE_SPI_CONST_CFG && SPI_M_SPI_WAKE_ENABLE_CONST)
        SPI_M_SpiSaveConfig();

    #elif (SPI_M_SCB_MODE_UART_CONST_CFG && SPI_M_UART_WAKE_ENABLE_CONST)
        SPI_M_UartSaveConfig();

    #else

        SPI_M_backup.enableState = (uint8) SPI_M_GET_CTRL_ENABLED;

        if(0u != SPI_M_backup.enableState)
        {
            SPI_M_Stop();
        }

    #endif /* defined (SPI_M_SCB_MODE_I2C_CONST_CFG) && (SPI_M_I2C_WAKE_ENABLE_CONST) */

#endif /* (SPI_M_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SPI_M_Wakeup
********************************************************************************
*
* Summary:
*  Prepares the component for the Active mode operation after exiting Deep Sleep.
*  The Enable wakeup from Sleep Mode option has an influence on this function
*  implementation.
*  This function should not be called after exiting Sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SPI_M_Wakeup(void)
{
#if(SPI_M_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SPI_M_SCB_WAKE_ENABLE_CHECK)
    {
        if(SPI_M_SCB_MODE_I2C_RUNTM_CFG)
        {
            SPI_M_I2CRestoreConfig();
        }
        else if(SPI_M_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SPI_M_EzI2CRestoreConfig();
        }
    #if(!SPI_M_CY_SCBIP_V1_I2C_ONLY)
        else if(SPI_M_SCB_MODE_SPI_RUNTM_CFG)
        {
            SPI_M_SpiRestoreConfig();
        }
        else if(SPI_M_SCB_MODE_UART_RUNTM_CFG)
        {
            SPI_M_UartRestoreConfig();
        }
    #endif /* (!SPI_M_CY_SCBIP_V1_I2C_ONLY) */
        else
        {
            /* Unknown mode: do nothing */
        }
    }
    else
    {
        if(0u != SPI_M_backup.enableState)
        {
            SPI_M_Enable();
        }
    }

#else

    #if (SPI_M_SCB_MODE_I2C_CONST_CFG  && SPI_M_I2C_WAKE_ENABLE_CONST)
        SPI_M_I2CRestoreConfig();

    #elif (SPI_M_SCB_MODE_EZI2C_CONST_CFG && SPI_M_EZI2C_WAKE_ENABLE_CONST)
        SPI_M_EzI2CRestoreConfig();

    #elif (SPI_M_SCB_MODE_SPI_CONST_CFG && SPI_M_SPI_WAKE_ENABLE_CONST)
        SPI_M_SpiRestoreConfig();

    #elif (SPI_M_SCB_MODE_UART_CONST_CFG && SPI_M_UART_WAKE_ENABLE_CONST)
        SPI_M_UartRestoreConfig();

    #else

        if(0u != SPI_M_backup.enableState)
        {
            SPI_M_Enable();
        }

    #endif /* (SPI_M_I2C_WAKE_ENABLE_CONST) */

#endif /* (SPI_M_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
