/*******************************************************************************
* File Name: msTimer_PM.c
* Version 2.80
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2017, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "msTimer.h"

static msTimer_backupStruct msTimer_backup;


/*******************************************************************************
* Function Name: msTimer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  msTimer_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void msTimer_SaveConfig(void) 
{
    #if (!msTimer_UsingFixedFunction)
        msTimer_backup.TimerUdb = msTimer_ReadCounter();
        msTimer_backup.InterruptMaskValue = msTimer_STATUS_MASK;
        #if (msTimer_UsingHWCaptureCounter)
            msTimer_backup.TimerCaptureCounter = msTimer_ReadCaptureCount();
        #endif /* Back Up capture counter register  */

        #if(!msTimer_UDB_CONTROL_REG_REMOVED)
            msTimer_backup.TimerControlRegister = msTimer_ReadControlRegister();
        #endif /* Backup the enable state of the Timer component */
    #endif /* Backup non retention registers in UDB implementation. All fixed function registers are retention */
}


/*******************************************************************************
* Function Name: msTimer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  msTimer_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void msTimer_RestoreConfig(void) 
{   
    #if (!msTimer_UsingFixedFunction)

        msTimer_WriteCounter(msTimer_backup.TimerUdb);
        msTimer_STATUS_MASK =msTimer_backup.InterruptMaskValue;
        #if (msTimer_UsingHWCaptureCounter)
            msTimer_SetCaptureCount(msTimer_backup.TimerCaptureCounter);
        #endif /* Restore Capture counter register*/

        #if(!msTimer_UDB_CONTROL_REG_REMOVED)
            msTimer_WriteControlRegister(msTimer_backup.TimerControlRegister);
        #endif /* Restore the enable state of the Timer component */
    #endif /* Restore non retention registers in the UDB implementation only */
}


/*******************************************************************************
* Function Name: msTimer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  msTimer_backup.TimerEnableState:  Is modified depending on the
*  enable state of the block before entering sleep mode.
*
*******************************************************************************/
void msTimer_Sleep(void) 
{
    #if(!msTimer_UDB_CONTROL_REG_REMOVED)
        /* Save Counter's enable state */
        if(msTimer_CTRL_ENABLE == (msTimer_CONTROL & msTimer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            msTimer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            msTimer_backup.TimerEnableState = 0u;
        }
    #endif /* Back up enable state from the Timer control register */
    msTimer_Stop();
    msTimer_SaveConfig();
}


/*******************************************************************************
* Function Name: msTimer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  msTimer_backup.enableState:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void msTimer_Wakeup(void) 
{
    msTimer_RestoreConfig();
    #if(!msTimer_UDB_CONTROL_REG_REMOVED)
        if(msTimer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                msTimer_Enable();
        } /* Do nothing if Timer was disabled before */
    #endif /* Remove this code section if Control register is removed */
}


/* [] END OF FILE */
